import java.math.BigInteger;
import java.text.*;

/** Cryptanalysis of the Enigma Machine.
 *
 * This program uses combinatorics and big integers
 * to analyze the cryptographic strength of the Enigma machine.
 *
 * Author: Charles Reid
 * Date: March 2017
 */

public class Combos {
    public static void main(String[] args) { 

		// This involves a double factorial. PREPARE YOUR ANUS
		boolean doBig = true;



		/////////////////////////////////
		// Git Ready

        DecimalFormat formatter = new DecimalFormat("#,###");

        BigInteger small_combos = new BigInteger("1");
        BigInteger big_combos = new BigInteger("1");

		/// Useful temp variable 
		BigInteger next;



		/////////////////////////////////
		// Constants

        int N = 10; // number of switchboard wires
        int P = 5;  // number of possible rotors
        int Q = 5;  // number of possible reflectors
        int R = 3;  // number of rotors
        int S = 26; // number of symbols




		/////////////////////////////////
		// Plugboard combinations

		BigInteger plugboard = new BigInteger("1");

		// S!/(S-2N)!
		BigInteger num = BigInteger.valueOf(1);
		for(int i=S; i>(S-2*N); i--) {
			next = BigInteger.valueOf(i);
			num = num.multiply(next);
		}

		// divided by 2^N
		BigInteger pdenom = BigInteger.valueOf(2);
		pdenom = pdenom.pow(N);

		// divided by N!
		for(int j = N; j>1; j--) { 
			next = BigInteger.valueOf(j);
			pdenom = pdenom.multiply(next);
		}

		plugboard = num.divide(pdenom);



		/////////////////////////////////
		// Rotor combinations

		BigInteger small_rotors = new BigInteger("1");
		BigInteger big_rotors = new BigInteger("1");

		// -----------------
		// Small case:
		// R rotors selected from P possible known rotors

		// Rotor wheel combinations
        for(int j=P; j>(P-R); j--) { 
            next = BigInteger.valueOf(j);
            small_rotors = small_rotors.multiply(next);
        }

        // Rotor wheel notch positions (assume 1 per wheel). 
        // Ignore left-most wheel.
        for(int k=0; k<(R-1); k++ ) {
            next = BigInteger.valueOf(S);
            small_rotors = small_rotors.multiply(next);
        }

        // Rotor wheel starting positions
        for(int k=0; k<R; k++ ) {
            next = BigInteger.valueOf(S);
            small_rotors = small_rotors.multiply(next);
        }

		// -----------------
		// Big case:
		// R rotors selected from S! possible rotors

		if(doBig) { 
			BigInteger s_rm1 = BigInteger.valueOf(S).pow(R-1);
			BigInteger sfact_r = BigInteger.valueOf(1);
			for(int j=S; j>=1; j--) { 
				next = BigInteger.valueOf(j);
				sfact_r = sfact_r.multiply(next);
			}
			sfact_r = sfact_r.pow(R);
			big_rotors = s_rm1.multiply(sfact_r);
		}

		/////////////////////////////////
		// Reflector combinations

		BigInteger small_reflector = BigInteger.valueOf(Q);
		BigInteger big_reflector = new BigInteger("1");

		// (S!)/((S/2)!)
		for(int j=S; j>(S/2); j--) {
			next = BigInteger.valueOf(j);
			big_reflector = big_reflector.multiply(next);
		}

		// divided by 2^N
		BigInteger rfldenom = BigInteger.valueOf(2);
		rfldenom = rfldenom.pow(N);

		if(doBig) { 
			big_reflector = big_reflector.divide(rfldenom);
		}



		/////////////////////////////////
		// Final combinations

		small_combos = small_combos.multiply(plugboard);
		small_combos = small_combos.multiply(small_rotors);
		small_combos = small_combos.multiply(small_reflector);

		if(doBig) {
			big_combos = big_combos.multiply(plugboard);
			big_combos = big_combos.multiply(big_rotors);
			big_combos = big_combos.multiply(big_reflector);
		}

        System.out.println("Final number of (small) Enigma combinations: ");
        System.out.println(formatter.format(small_combos));

		if(doBig) { 
			System.out.println("Final number of (big) Enigma combinations: ");
        	System.out.println(formatter.format(big_combos));
		}

    }
}
